#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "attr.h"
#include "tags.h"
#include "parser.h"


void parseTags(char* tag, tagDetails* details)
{
  char *str;
  char *buf = (char*)malloc(sizeof(char)*(strlen(tag)+1));
  
  strcpy(buf, tag);
  str = strtok(buf, "\n\t=! ");
  if(str[0] == '\0')
    str = strtok(NULL, "\n\t=! "); 
  details->tagName = (char*)malloc(sizeof(char)*(strlen(str)+1));
  strcpy(details->tagName, str);
  details->attr = NULL;
  while((str = strtok(NULL, "\n\t=! ")) != NULL)
    {
      addAttr(details, str);
    }
  free(buf);
}

char** getTags(FILE *testFile, int *numTags, FILE *log)
{
  char **tags = 0;
  char buf[64000];
  int cnt = 0, size = 0;
  
  while(!feof(testFile)) 
    {
      int i;
      if(cnt == size)
	{
	  tags = (char **) realloc(tags, (size+512)*sizeof(char*));
	  size += 512;
	  if(!tags)
	    {         
	      fprintf(stderr, "Unable to allocate memory... quitting\n");
	      fprintf(log,">>>PANIC<<< Cannot allocate memory to store any TAG\n");
	      exit(99);
	    } /* if */
	  for(i = cnt; i < size; i++)
	    tags[i] = (char*) malloc(512*sizeof(char));      
	}
      /* Read stream till '<' and then flush the '<' from the stream.*/
      fscanf(testFile,"%[^<]", buf); 
      fgetc(testFile);
      
      /* Now read the tag till '>' */
      fscanf(testFile,"%[^>]", buf);
      /* 
       * If it is a valid string, then add it, also takes care of comments
       * and Javascript embedded in the comments. It will not remove the 
       * tag for existence of the script though... which is sufficient for 
       * our use.
       */
      if(!strstr(buf, "--") && !isspace((int)buf[0]))
	{
	  /* Okay we have the tag now. Increment the tag count. Allocate memory
	     // to save the tag. Finally save the tag.*/
	  if(tags[cnt]) 
	    {
	      strcpy(tags[cnt], buf);
	      cnt++;
	    } /* if */ 
	  else            /* Again could not create memory to store the TAG.*/
	    {             /* So free any memory allocated so far and return.*/
	      int i = 0;   
	      fprintf(stderr , "Am entering else region... why?\n");
	      while(i < cnt) 
		{
		  free(tags[i]);
		}
	      free(tags);
	      fprintf(stderr, "Unable to allocate memory... quitting\n");
	      fprintf(log,">>>PANIC<<< Cannot allocate memory to store some TAGs\n");
	      exit(99);
	    }
	}
      
      /* 
       * flush the > sign from the stream. This is need for the last one where 
       * we will not hit EOF otherwise. For all others, this will be ignored.
       * The last one ABSOLUTELY REQUIRES this.
       */
      fgetc(testFile);
      
      /* 
       * Need this to strip out the JavaScript. We could otherwise have 
       * problems because the Javascript may have the > sign for comparison. 
       */
      /* if((strstr(buf, "SCRIPT") || strstr(buf, "script")) &&  */
      /* 	 (!strstr(buf,"/SCRIPT") && !strstr(buf, "/script"))) */
      /* 	{ */
      /* 	  fscanf(testFile, "%[^->]", buf); */
      /* 	} */
    }
  
  /* Let the user know the number of tags got */
  *numTags = cnt;
  
  return tags;
}
