#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "html.h"
#include "stack.h"
#include "attr.h"
#include "tags.h"

extern void stripSpace(char *, int);

struct _tag tagList[] =
{
    {"html",       (VERS_ALL|VERS_FRAMES)},
    {"head",       (VERS_ALL|VERS_FRAMES)},
    {"title",      (VERS_ALL|VERS_FRAMES)},
    {"base",       (VERS_ALL|VERS_FRAMES)},
    {"link",       (VERS_ALL|VERS_FRAMES)},
    {"meta",       (VERS_ALL|VERS_FRAMES)},
    {"style",      (VERS_FROM32|VERS_FRAMES)},
    {"script",     (VERS_FROM32|VERS_FRAMES)},
    {"server",     VERS_NETSCAPE,},
    {"body",       VERS_ALL},
    {"frameset",   VERS_FRAMES},
    {"p",          VERS_ALL},
    {"h1",         VERS_ALL},
    {"h2",         VERS_ALL},
    {"h3",         VERS_ALL},
    {"h4",         VERS_ALL},
    {"h5",         VERS_ALL},
    {"h6",         VERS_ALL},
    {"ul",         VERS_ALL},
    {"ol",         VERS_ALL},
    {"dl",         VERS_ALL},
    {"dir",        VERS_LOOSE},
    {"menu",       VERS_LOOSE},
    {"pre",        VERS_ALL},
    {"listing",    VERS_ALL},
    {"xmp",        VERS_ALL},
    {"plaintext",  VERS_ALL},
    {"address",    VERS_ALL},
    {"blockquote", VERS_ALL},
    {"form",       VERS_ALL},
    {"isindex",    VERS_LOOSE},
    {"fieldset",   VERS_HTML40},
    {"table",      VERS_FROM32},
    {"hr",         VERS_ALL},
    {"div",        VERS_FROM32},
    {"multicol",   VERS_NETSCAPE},
    {"nosave",     VERS_NETSCAPE},
    {"layer",      VERS_NETSCAPE},
    {"ilayer",     VERS_NETSCAPE},
    {"nolayer",    VERS_NETSCAPE},
    {"align",      VERS_NETSCAPE},
    {"center",     VERS_LOOSE},
    {"ins",        VERS_HTML40},
    {"del",        VERS_HTML40},
    {"li",         VERS_ALL},
    {"dt",         VERS_ALL},
    {"dd",         VERS_ALL},
    {"caption",    VERS_FROM32},
    {"colgroup",   VERS_HTML40},
    {"col",        VERS_HTML40},
    {"thead",      VERS_HTML40},
    {"tfoot",      VERS_HTML40},
    {"tbody",      VERS_HTML40},
    {"tr",         VERS_FROM32},
    {"td",         VERS_FROM32},
    {"th",         VERS_FROM32},
    {"q",          VERS_HTML40},
    {"a",          VERS_ALL},
    {"br",         VERS_ALL},
    {"img",        VERS_ALL},
    {"object",     VERS_HTML40},
    {"applet",     VERS_LOOSE},
    {"servlet",    VERS_SUN},
    {"param",      VERS_FROM32},
    {"embed",      VERS_NETSCAPE},
    {"noembed",    VERS_NETSCAPE},
    {"iframe",     VERS_HTML40_LOOSE},
    {"frame",      VERS_FRAMES},
    {"noframes",   VERS_IFRAMES},
    {"noscript",   (VERS_FRAMES|VERS_HTML40)},
    {"b",          VERS_ALL},
    {"i",          VERS_ALL},
    {"u",          VERS_LOOSE},
    {"tt",         VERS_ALL},
    {"s",          VERS_LOOSE},
    {"strike",     VERS_LOOSE},
    {"big",        VERS_FROM32},
    {"small",      VERS_FROM32},
    {"sub",        VERS_FROM32},
    {"sup",        VERS_FROM32},
    {"em",         VERS_ALL},
    {"strong",     VERS_ALL},
    {"dfn",        VERS_ALL},
    {"code",       VERS_ALL},
    {"samp",       VERS_ALL},
    {"kbd",        VERS_ALL},
    {"var",        VERS_ALL},
    {"cite",       VERS_ALL},
    {"abbr",       VERS_HTML40},
    {"acronym",    VERS_HTML40},
    {"span",       VERS_FROM32},
    {"blink",      VERS_PROPRIETARY},
    {"nobr",       VERS_PROPRIETARY},
    {"wbr",        VERS_PROPRIETARY},
    {"marquee",    VERS_MICROSOFT},
    {"bgsound",    VERS_MICROSOFT},
    {"comment",    VERS_MICROSOFT},
    {"spacer",     VERS_NETSCAPE},
    {"keygen",     VERS_NETSCAPE},
    {"nolayer",    VERS_NETSCAPE},
    {"ilayer",     VERS_NETSCAPE},
    {"map",        VERS_FROM32},
    {"area",       VERS_ALL},
    {"input",      VERS_ALL},
    {"select",     VERS_ALL},
    {"option",     VERS_ALL},
    {"optgroup",   VERS_HTML40},
    {"textarea",   VERS_ALL},
    {"label",      VERS_HTML40},
    {"legend",     VERS_HTML40},
    {"button",     VERS_HTML40},
    {"basefont",   VERS_LOOSE},
    {"font",       VERS_LOOSE},
    {"bdo",        VERS_HTML40},
	{"doctype",	   VERS_ALL},	//added by me, not in original list...
	/* this must be the final entry */
    {NULL,         0}
};

void strLower(char *src, char *dest)
{
	while(*src != '\0')
    {
		*dest = tolower((int)*src);
		src++; dest++;
    }
	*dest = '\0';
}

void strUpper(char *src, char *dest)
{
	while(*src != '\0')
    {
		*dest = toupper((int)*src);
		src++; dest++;
    }
	*dest = '\0';
}

void addAttr(tagDetails *tag, char *name)
{	
	ATTR *temp;
	temp = tag->attr;
	
	if(tag->attr == NULL)
	{
		tag->attr = (ATTR*) malloc(sizeof(ATTR));
		temp = tag->attr;
	}
	else
    {
		while(temp->next != NULL)
			temp = temp->next;
		temp->next = (ATTR*) malloc(sizeof(ATTR));
		temp = temp->next;
    }
	temp->name  = (char*) malloc(sizeof(char)*(strlen(name)+1));
	temp->next = NULL;
	strcpy(temp->name, name);
}

void delAttr(tagDetails *tag)
{
	ATTR *temp1, *temp2;
	temp1 = temp2 = tag->attr;
	while(temp1)
    {
		temp2 = temp1->next;
		free(temp1->name);
		free(temp1);
		temp1 = temp2;
    }
	tag->attr = NULL;
}

void printAttr(tagDetails * tag)
{
	ATTR *temp;
	
	temp = tag->attr;
	while(temp != 0)
    {
		printf("->%s", temp->name);
		temp = temp->next;
    }
}

int findTag(char *s)
{
	int index = -1, i = 0;
	char *temp;
	
	temp = (char*)malloc((strlen(s)+1) * sizeof(char));
	if(s[0] == '/')
		s++;
	stripSpace(s, strlen(s)+1);
	strLower(s, temp);
	while(tagList[i].name != NULL)
    {
		if(strcmp(temp, tagList[i].name) == 0)
		{
			index = i;
			break;
		}
		i++;
    }
	free(temp);
	return index;
}

int checkTags(tagDetails* details, int len, FILE *log)
{
	int cnt = 0, i;
	int flag = 0;
	
	while(cnt < len)
	{
		i = findTag(details[cnt].tagName);
		if( i != -1 )
		{
			switch(tagList[i].versions)
			{
			case VERS_PROPRIETARY:
				fprintf(log,"Found proprietary tag... %s.\n", details[cnt].tagName);
				flag = 1;
				break;
			case VERS_FRAMES:
				fprintf(log,"Found frame's tag... %s.\n", details[cnt].tagName);
				flag = 1;
				break;
			case VERS_SUN:
				fprintf(log,"Found tag supported by Sun only... %s.\n", details[cnt].tagName);
				flag = 1;
				break;
			case VERS_MICROSOFT:
				fprintf(log,"Found tag supported by Microsoft... %s.\n", details[cnt].tagName);
				flag = 1;
				break;
			case VERS_NETSCAPE: 
				fprintf(log,"Found tag supported by Netscape... %s.\n", details[cnt].tagName);
				flag = 1;
				break;
			case VERS_LOOSE:
				fprintf(log,"Found \"loose\" html tag... %s.\n", details[cnt].tagName);
				flag = 1;
				break;
			}
		}
		else
		{
			fprintf(log,">>>FAULT<<< Invalid HTML tag... %s\n", details[cnt].tagName);
			flag = 1;
		}
		cnt++;
	}
	return flag;
}	

int checkClosing(tagDetails *tags, int len, FILE *log)
{
	int i = 0, flag = 0;
	stack *st = (stack*)malloc(sizeof(stack));
	
	initStack(st);
	
	while(i < len)
	{
		char *temp = tags[i].tagName;
		char *t;
		
		if((strstr(temp,"html") != 0) || (strstr(temp,"HTML") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"title") != 0) || (strstr(temp,"TITLE") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"meta") != 0) || (strstr(temp,"META") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"head") != 0) || (strstr(temp,"HEAD") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"body") != 0) || (strstr(temp,"BODY") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"tbody") != 0) || (strstr(temp,"TBODY") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"p") != 0) || (strstr(temp,"P") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"li") != 0) || (strstr(temp,"LI") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"dh") != 0) || (strstr(temp,"DH") != 0))
		{
			i++;
			continue;
		}
		if((strstr(temp,"dd") != 0) || (strstr(temp,"DD") != 0))
		{
			i++;
			continue;
		}
		if((t = strchr(temp,'/')) != NULL)
		{
			char buf[128];
			
			t++;
			while(!isEmpty(st) && (strcmp(t, top(st)) != 0))
			{
				flag = 1;
				pop(st, buf);
				fprintf(log, "Unmatched opening tags... %s\n", buf);
			}
			pop(st, buf);
			i++;
			continue;
		}
		push(st, temp);
		i++;
	}
	free(st);

	return flag;
}

void checkProbs(tagDetails* tags, int len, int *test, FILE *log)
{
	int i = 0;
	while(i < len)
	{
		if(!strcmp(tags[i].tagName, "SCRIPT") || !strcmp(tags[i].tagName, "SCRIPT"))
		{
			fprintf(log, "Checking in tag... SCRIPT\n");
			test[JSCRIPT] = 1;
		}
		if(!strcmp(tags[i].tagName, "APPLET") || !strcmp(tags[i].tagName, "APPLET"))
		{
			fprintf(log, "Checking in tag... APPLET\n");
			test[JAVA] = 1;
		}
		if(!strcmp(tags[i].tagName, "OBJECT") || !strcmp(tags[i].tagName, "OBJECT"))
		{
			fprintf(log, "Checking in tag... OBJECT\n");
		}
		if(!strcmp(tags[i].tagName, "FONT") || !strcmp(tags[i].tagName, "FONT"))
		{
			ATTR *temp;
			
			fprintf(log, "Checking in tag... FONT\n");
			temp = tags[i].attr;
			while(temp != NULL)
			{
				if(!strcmp(temp->name, "SIZE") || !strcmp(temp->name, "size"))
					test[FONT] = 1;
				temp = temp->next;
			}
		}
		if(!strcmp(tags[i].tagName, "IMG") || !strcmp(tags[i].tagName, "img"))
		{
			ATTR *temp;
			
			fprintf(log, "Checking in tag... IMG\n");
			temp = tags[i].attr;
			test[IMAGE] = 1;
			while(temp != NULL)
			{
				if(!strcmp(temp->name, "ALT") || !strcmp(temp->name, "ALT"))
					test[IMAGE] = 0;
				temp = temp->next;
			}
		}
		if(!strcmp(tags[i].tagName, "TABLE") || !strcmp(tags[i].tagName, "table"))
		{
			ATTR *temp;
			
			fprintf(log, "Checking in tag... TABLE\n");
			temp = tags[i].attr;
			while(temp != NULL)
			{
				if(strstr(temp->name, "color") || strstr(temp->name, "COLOR"))
					test[IMAGE] = 1;
				temp = temp->next;
			}
		}
		i++;
	}
	return;
}
